@@ -20,25 +20,86 @@
  *
  * From: NetBSD: print-arcnet.c,v 1.2 2000/04/24 13:02:28 itojun Exp
  */
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-arcnet.c,v 1.20 2005-04-06 21:32:38 mcr Exp $ (LBL)";
-#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <tcpdump-stdinc.h>
+#include <netdissect-stdinc.h>
 
-#include <stdio.h>
-#include <pcap.h>
-
-#include "interface.h"
+#include "netdissect.h"
 #include "extract.h"
-#include "arcnet.h"
 
-static int arcnet_encap_print(u_char arctype, const u_char *p,
+/*
+ * from: NetBSD: if_arc.h,v 1.13 1999/11/19 20:41:19 thorpej Exp
+ */
+
+/*
+ * Structure of a 2.5MB/s Arcnet header on the BSDs,
+ * as given to interface code.
+ */
+struct	arc_header {
+	uint8_t  arc_shost;
+	uint8_t  arc_dhost;
+	uint8_t  arc_type;
+	/*
+	 * only present for newstyle encoding with LL fragmentation.
+	 * Don't use sizeof(anything), use ARC_HDR{,NEW}LEN instead.
+	 */
+	uint8_t  arc_flag;
+	uint16_t arc_seqid;
+
+	/*
+	 * only present in exception packets (arc_flag == 0xff)
+	 */
+	uint8_t  arc_type2;	/* same as arc_type */
+	uint8_t  arc_flag2;	/* real flag value */
+	uint16_t arc_seqid2;	/* real seqid value */
+};
+
+#define	ARC_HDRLEN		3
+#define	ARC_HDRNEWLEN		6
+#define	ARC_HDRNEWLEN_EXC	10
+
+/* RFC 1051 */
+#define	ARCTYPE_IP_OLD		240	/* IP protocol */
+#define	ARCTYPE_ARP_OLD		241	/* address resolution protocol */
+
+/* RFC 1201 */
+#define	ARCTYPE_IP		212	/* IP protocol */
+#define	ARCTYPE_ARP		213	/* address resolution protocol */
+#define	ARCTYPE_REVARP		214	/* reverse addr resolution protocol */
+
+#define	ARCTYPE_ATALK		221	/* Appletalk */
+#define	ARCTYPE_BANIAN		247	/* Banyan Vines */
+#define	ARCTYPE_IPX		250	/* Novell IPX */
+
+#define ARCTYPE_INET6		0xc4	/* IPng */
+#define ARCTYPE_DIAGNOSE	0x80	/* as per ANSI/ATA 878.1 */
+
+/*
+ * Structure of a 2.5MB/s Arcnet header on Linux.  Linux has
+ * an extra "offset" field when given to interface code, and
+ * never presents packets that look like exception frames.
+ */
+struct	arc_linux_header {
+	uint8_t  arc_shost;
+	uint8_t  arc_dhost;
+	uint16_t arc_offset;
+	uint8_t  arc_type;
+	/*
+	 * only present for newstyle encoding with LL fragmentation.
+	 * Don't use sizeof(anything), use ARC_LINUX_HDR{,NEW}LEN
+	 * instead.
+	 */
+	uint8_t  arc_flag;
+	uint16_t arc_seqid;
+};
+
+#define	ARC_LINUX_HDRLEN	5
+#define	ARC_LINUX_HDRNEWLEN	8
+
+static int arcnet_encap_print(netdissect_options *, u_char arctype, const u_char *p,
     u_int length, u_int caplen);
 
 static const struct tok arctypemap[] = {
@@ -56,7 +117,8 @@ static const struct tok arctypemap[] = {
 };
 
 static inline void
-arcnet_print(const u_char *bp, u_int length, int phds, int flag, u_int seqid)
+arcnet_print(netdissect_options *ndo, const u_char *bp, u_int length, int phds,
+             int flag, u_int seqid)
 {
 	const struct arc_header *ap;
 	const char *arctypename;
@@ -65,40 +127,40 @@ arcnet_print(const u_char *bp, u_int length, int phds, int flag, u_int seqid)
 	ap = (const struct arc_header *)bp;
 
 
-	if (qflag) {
-		(void)printf("%02x %02x %d: ",
+	if (ndo->ndo_qflag) {
+		ND_PRINT((ndo, "%02x %02x %d: ",
 			     ap->arc_shost,
 			     ap->arc_dhost,
-			     length);
+			     length));
 		return;
 	}
 
 	arctypename = tok2str(arctypemap, "%02x", ap->arc_type);
 
 	if (!phds) {
-		(void)printf("%02x %02x %s %d: ",
+		ND_PRINT((ndo, "%02x %02x %s %d: ",
 			     ap->arc_shost, ap->arc_dhost, arctypename,
-			     length);
+			     length));
 			     return;
 	}
 
 	if (flag == 0) {
-		(void)printf("%02x %02x %s seqid %04x %d: ",
+		ND_PRINT((ndo, "%02x %02x %s seqid %04x %d: ",
 			ap->arc_shost, ap->arc_dhost, arctypename, seqid,
-			length);
+			length));
 			return;
 	}
 
 	if (flag & 1)
-		(void)printf("%02x %02x %s seqid %04x "
+		ND_PRINT((ndo, "%02x %02x %s seqid %04x "
 			"(first of %d fragments) %d: ",
 			ap->arc_shost, ap->arc_dhost, arctypename, seqid,
-			(flag + 3) / 2, length);
+			(flag + 3) / 2, length));
 	else
-		(void)printf("%02x %02x %s seqid %04x "
+		ND_PRINT((ndo, "%02x %02x %s seqid %04x "
 			"(fragment %d) %d: ",
 			ap->arc_shost, ap->arc_dhost, arctypename, seqid,
-			flag/2 + 1, length);
+			flag/2 + 1, length));
 }
 
 /*
@@ -108,7 +170,7 @@ arcnet_print(const u_char *bp, u_int length, int phds, int flag, u_int seqid)
  * is the number of bytes actually captured.
  */
 u_int
-arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
+arcnet_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char *p)
 {
 	u_int caplen = h->caplen;
 	u_int length = h->len;
@@ -118,8 +180,8 @@ arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	u_int seqid = 0;
 	u_char arc_type;
 
-	if (caplen < ARC_HDRLEN) {
-		printf("[|arcnet]");
+	if (caplen < ARC_HDRLEN || length < ARC_HDRLEN) {
+		ND_PRINT((ndo, "[|arcnet]"));
 		return (caplen);
 	}
 
@@ -139,16 +201,16 @@ arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	}
 
 	if (phds) {
-		if (caplen < ARC_HDRNEWLEN) {
-			arcnet_print(p, length, 0, 0, 0);
-			printf("[|phds]");
+		if (caplen < ARC_HDRNEWLEN || length < ARC_HDRNEWLEN) {
+			arcnet_print(ndo, p, length, 0, 0, 0);
+			ND_PRINT((ndo, "[|phds]"));
 			return (caplen);
 		}
 
 		if (ap->arc_flag == 0xff) {
-			if (caplen < ARC_HDRNEWLEN_EXC) {
-				arcnet_print(p, length, 0, 0, 0);
-				printf("[|phds extended]");
+			if (caplen < ARC_HDRNEWLEN_EXC || length < ARC_HDRNEWLEN_EXC) {
+				arcnet_print(ndo, p, length, 0, 0, 0);
+				ND_PRINT((ndo, "[|phds extended]"));
 				return (caplen);
 			}
 			flag = ap->arc_flag2;
@@ -162,8 +224,8 @@ arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	}
 
 
-	if (eflag)
-		arcnet_print(p, length, phds, flag, seqid);
+	if (ndo->ndo_eflag)
+		arcnet_print(ndo, p, length, phds, flag, seqid);
 
 	/*
 	 * Go past the ARCNET header.
@@ -179,8 +241,8 @@ arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
 		return (archdrlen);
 	}
 
-	if (!arcnet_encap_print(arc_type, p, length, caplen))
-		default_print(p, caplen);
+	if (!arcnet_encap_print(ndo, arc_type, p, length, caplen))
+		ND_DEFAULTPRINT(p, caplen);
 
 	return (archdrlen);
 }
@@ -196,7 +258,7 @@ arcnet_if_print(const struct pcap_pkthdr *h, const u_char *p)
  * extra "offset" field between the src/dest and packet type.
  */
 u_int
-arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
+arcnet_linux_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char *p)
 {
 	u_int caplen = h->caplen;
 	u_int length = h->len;
@@ -205,8 +267,8 @@ arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	int archdrlen = 0;
 	u_char arc_type;
 
-	if (caplen < ARC_LINUX_HDRLEN) {
-		printf("[|arcnet]");
+	if (caplen < ARC_LINUX_HDRLEN || length < ARC_LINUX_HDRLEN) {
+		ND_PRINT((ndo, "[|arcnet]"));
 		return (caplen);
 	}
 
@@ -216,8 +278,8 @@ arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	switch (arc_type) {
 	default:
 		archdrlen = ARC_LINUX_HDRNEWLEN;
-		if (caplen < ARC_LINUX_HDRNEWLEN) {
-			printf("[|arcnet]");
+		if (caplen < ARC_LINUX_HDRNEWLEN || length < ARC_LINUX_HDRNEWLEN) {
+			ND_PRINT((ndo, "[|arcnet]"));
 			return (caplen);
 		}
 		break;
@@ -228,8 +290,8 @@ arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
 		break;
 	}
 
-	if (eflag)
-		arcnet_print(p, length, 0, 0, 0);
+	if (ndo->ndo_eflag)
+		arcnet_print(ndo, p, length, 0, 0, 0);
 
 	/*
 	 * Go past the ARCNET header.
@@ -238,8 +300,8 @@ arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	caplen -= archdrlen;
 	p += archdrlen;
 
-	if (!arcnet_encap_print(arc_type, p, length, caplen))
-		default_print(p, caplen);
+	if (!arcnet_encap_print(ndo, arc_type, p, length, caplen))
+		ND_DEFAULTPRINT(p, caplen);
 
 	return (archdrlen);
 }
@@ -253,36 +315,34 @@ arcnet_linux_if_print(const struct pcap_pkthdr *h, const u_char *p)
 
 
 static int
-arcnet_encap_print(u_char arctype, const u_char *p,
+arcnet_encap_print(netdissect_options *ndo, u_char arctype, const u_char *p,
     u_int length, u_int caplen)
 {
 	switch (arctype) {
 
 	case ARCTYPE_IP_OLD:
 	case ARCTYPE_IP:
-	        ip_print(gndo, p, length);
+	        ip_print(ndo, p, length);
 		return (1);
 
-#ifdef INET6
 	case ARCTYPE_INET6:
-		ip6_print(gndo, p, length);
+		ip6_print(ndo, p, length);
 		return (1);
-#endif /*INET6*/
 
 	case ARCTYPE_ARP_OLD:
 	case ARCTYPE_ARP:
 	case ARCTYPE_REVARP:
-		arp_print(gndo, p, length, caplen);
+		arp_print(ndo, p, length, caplen);
 		return (1);
 
 	case ARCTYPE_ATALK:	/* XXX was this ever used? */
-		if (vflag)
-			fputs("et1 ", stdout);
-		atalk_print(p, length);
+		if (ndo->ndo_vflag)
+			ND_PRINT((ndo, "et1 "));
+		atalk_print(ndo, p, length);
 		return (1);
 
 	case ARCTYPE_IPX:
-		ipx_print(p, length);
+		ipx_print(ndo, p, length);
 		return (1);
 
 	default:
